//
//  ViewController.m
//  BackendlessTest
//
//  Created by Matej Trbara on 23.11.2020..
//

#import "ViewController.h"
#import <Backendless-Swift.h>

@interface ViewController() {
    Channel * _defaultChannel;
}

@property (unsafe_unretained) IBOutlet NSTextView *outputField;

@end

@implementation ViewController

- (void)viewDidLoad {
    _defaultChannel = [[Channel alloc] initWithChannelName:@"default"];
}

- (void)viewDidAppear {
//    [self validateUserToken];
}

-(void) login {
    [self appendToConsole:@"Started login..."];
    [Backendless.shared.userService loginWithIdentity:@"test@test.com" password:@"test" responseHandler:^(BackendlessUser *loggedInUser) {
        [self appendToConsole:@"User has been logged in."];
    } errorHandler:^(Fault *fault) {
        [self appendToConsole:[NSString stringWithFormat:@"login error: %@", fault.message]];
    }];
}

-(void) logout {
    [self appendToConsole:@"Started logout..."];
    [Backendless.shared.userService logoutWithResponseHandler:^{
        [self appendToConsole:@"User has been logged out"];
    } errorHandler:^(Fault *fault) {
        [self appendToConsole:[NSString stringWithFormat:@"logout error: %@", fault.message]];
    }];
}

-(void)validateUserToken {
    [self appendToConsole:@"Started user token validation..."];
    [Backendless.shared.userService isValidUserTokenWithResponseHandler:^(BOOL isValid) {
        if(isValid) {
            [self appendToConsole:@"User token is valid. User logged in."];
        } else {
            [self appendToConsole:@"User token is invalid. User not logged in."];
        }
    } errorHandler:^(Fault *fault) {
        [self appendToConsole:[NSString stringWithFormat:@"validate user token error: %@", fault.message]];
    }];
}

- (void)joinDefaultChannel {
    [self appendToConsole:@"Started default channel join..."];
    if(_defaultChannel.isJoined) {
        [self appendToConsole:@"Default channel already joined."];
    } else {
        [_defaultChannel join];
        __weak ViewController * weakSelf = self;
        [_defaultChannel addConnectListenerWithResponseHandler:^{
            [weakSelf appendToConsole:@"Default channel joined."];
        } errorHandler:^(Fault * _Nonnull fault) {
            [weakSelf appendToConsole:[NSString stringWithFormat:@"default channel join error: %@", fault.message]];
        }];
    }
}

- (void) isDefaultChannelJoined {
    [self appendToConsole:[NSString stringWithFormat:@"Default channel is joined: %d", _defaultChannel.isJoined]];
}

- (void)leaveDefaultChannel {
    [_defaultChannel leave];
    [self appendToConsole:@"Default channel should be left. Check with button 'Is default channel joined.'"];
}

- (void)addStringMessageListener {
    __weak ViewController * weakSelf = self;
    [_defaultChannel addStringMessageListenerWithResponseHandler:^(NSString * _Nonnull msg) {
        [weakSelf appendToConsole:[NSString stringWithFormat:@"Received message from default channel: %@", msg]];
    } errorHandler:^(Fault * _Nonnull fault) {
        [weakSelf appendToConsole:[NSString stringWithFormat:@"Received message from default channel error: %@", fault.message]];
    }];
    [self appendToConsole:@"String message listener added."];
}

- (void)removeAllMessageListeners {
    [_defaultChannel removeMessageListeners];
    [self appendToConsole:@"Message listeners removed."];
}

- (void)appendToConsole:(NSString*)string {
    _outputField.string = [_outputField.string stringByAppendingFormat:@"%@\n", string];
}


- (IBAction)onLoginclick:(id)sender {
    [self login];
}

- (IBAction)onLogoutClick:(id)sender {
    [self logout];
}
- (IBAction)onValidateuserTokenClick:(id)sender {
    [self validateUserToken];
}

- (IBAction)onJoinDefaultChannelClick:(id)sender {
    [self joinDefaultChannel];
}

- (IBAction)onLeaveDefaultChannelClick:(id)sender {
    [self leaveDefaultChannel];
}

- (IBAction)onIsDefaultChannelJoinedClick:(id)sender {
    [self isDefaultChannelJoined];
}

- (IBAction)onAddStringMessageListenerClick:(id)sender {
    [self addStringMessageListener];
}

- (IBAction)onRemoveAllMessageListenersClick:(id)sender {
    [self removeAllMessageListeners];
}


@end
